MODULE sbcdcy
   !!======================================================================
   !!                    ***  MODULE  sbcdcy  ***
   !! Ocean forcing:  compute the diurnal cycle
   !!======================================================================
   !! History : 8.2  !  2005-02  (D. Bernie)  Original code
   !!           9.0  !  2006-02  (S. Masson, G. Madec)  adaptation to OPA9
   !!           3.1  !  2009-07  (J.M. Molines)  adaptation to nemo3.1
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!  sbc_dcy : compute solar flux at kt from daily mean, taking 
   !!            diurnal cycle into account
   !!----------------------------------------------------------------------
   USE oce              ! ocean dynamics and tracers
   USE phycst           ! ocean physics
   USE dom_oce          ! ocean space and time domain
   USE in_out_manager   ! I/O manager

   IMPLICIT NONE
   PRIVATE
   INTEGER                      ::   idayqsr                                            ! day when parameters were computed
   REAL(wp), DIMENSION(jpi,jpj) ::   zaaa, zbbb, zccc, zab, ztmd, zdawn, zdusk, zscal   ! parameters to compute the diurnal cycle
   REAL(wp), DIMENSION(jpi,jpj) ::   qsr_daily                                          ! to hold daily mean QSR
  
   PUBLIC sbc_dcy       ! routine called by sbc

   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.3 , LOCEAN-IPSL (2010) 
   !! $Id$ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

      SUBROUTINE sbc_dcy( kt, pqsr )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE sbc_dcy  ***
      !!
      !! ** Purpose : introduce a diurnal cycle of qsr from daily values
      !!
      !! ** Method  : see Appendix A of 
      !!              Bernie, DJ, Guilyardi, E, Madec, G, Slingo, JM and Woolnough, SJ
      !!              Impact of resolving the diurnal cycle in an ocean--atmosphere GCM. Part 1: a diurnally forced OGCM
      !!              Climate Dynamics 29:6, 575-590 (2007)
      !!
      !! ** Action  : redistribute daily QSR on each time step following the diurnal cycle
      !!----------------------------------------------------------------------
      INTEGER,                      INTENT( in    ) ::   kt     ! ocean time-step index
      REAL(wp), DIMENSION(jpi,jpj), INTENT( inout ) ::   pqsr   ! QSR flux with diurnal cycle
      !!
      INTEGER  ::   ji, jj                                      ! dummy loop indices
      REAL(wp) ::   fintegral, pt1, pt2, paaa, pbbb, pccc       ! 
      REAL(wp) ::   ztwopi, zinvtwopi, zconvrad 
      REAL(wp) ::   zlo, zup, zlousd, zupusd
      REAL(wp) ::   zdsws, zdecrad, ztx
      REAL(wp) ::   ztmp, ztmp1, ztmp2, ztest
      !!---------------------------------------------------------------------

      !----------------------------------------------------------------------
      ! statement functions

      fintegral(pt1, pt2, paaa, pbbb, pccc) =                           &
         &   paaa * pt2 + zinvtwopi * pbbb * SIN(pccc + ztwopi * pt2)   &
         & - paaa * pt1 - zinvtwopi * pbbb * SIN(pccc + ztwopi * pt1)
      !----------------------------------------------------------------------

      ! Initialization
      ! --------------
      ztwopi    = 2. * rpi
      zinvtwopi = 1. / ztwopi
      zconvrad  = ztwopi / 360.

      ! When are we during the day (from 0 to 1)
      zlo = MOD( rdt / rday * REAL( kt-nit000, wp ), 1.)
      zup = zlo + rdt / rday

      !                                          
      IF (kt == nit000) THEN                    
         !
         IF(lwp) THEN
            WRITE(numout,*)
            WRITE(numout,*) 'sbc_dcy : introduce diurnal cycle from daily mean qsr'
            WRITE(numout,*) '~~~~~~~'
            WRITE(numout,*)
         ENDIF
         idayqsr = 0
         ! Compute C needed to compute the time integral of the diurnal cycle
         zccc(:,:) = zconvrad * glamt(:,:) - rpi
         ! time of midday
         ztmd(:,:) = 0.5 - glamt(:,:) / 360.
         ztmd(:,:) = MOD((ztmd(:,:) + 1.), 1.)
      ENDIF

      ! If this is a new day, we have to update the dawn, dusk and scaling function  
      !----------------------
    
      !     2.1 dawn and dusk  

      ! nday is the number of days since the beginning of the current month 
      IF( idayqsr /= nday ) THEN 
         ! save the day of the year and the daily mean of qsr
         idayqsr = nday 
         ! number of days since the previous winter solstice (supposed to be always 21 December)         
         zdsws = 11 + nday_year
         ! declination of the earths orbit
         zdecrad = (-23.5 * zconvrad) * COS( zdsws * ztwopi / REAL(nyear_len(1),wp) )
         ! save the daily QSR for nest hours of the day
         qsr_daily(:,:) = pqsr(:,:)
         ! Compute A and B needed to compute the time integral of the diurnal cycle
        
         DO jj = 1, jpj
            DO ji = 1, jpi
               ztmp = zconvrad * gphit(ji,jj)
               zaaa(ji,jj) = SIN(ztmp) * SIN(zdecrad)
               zbbb(ji,jj) = COS(ztmp) * COS(zdecrad)
            END DO  
         END DO  

         ! Compute the time of dawn and dusk

         ! zab to test if the day time is equal to 0, less than 24h of full day        
         zab(:,:) = -zaaa(:,:) / zbbb(:,:)
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF ( ABS(zab(ji,jj)) < 1 ) THEN
         ! day duration is less than 24h
         ! When is it night?
                  ztx = zinvtwopi * (ACOS(zab(ji,jj)) - zccc(ji,jj))
                  ztest = -zbbb(ji,jj) * SIN( zccc(ji,jj) + ztwopi * ztx )
         ! is it dawn or dusk?
                  IF ( ztest > 0 ) THEN
                     zdawn(ji,jj) = ztx
                     zdusk(ji,jj) = ztmd(ji,jj) + ( ztmd(ji,jj) - zdawn(ji,jj) )
                  ELSE
                     zdusk(ji,jj) = ztx
                     zdawn(ji,jj) = ztmd(ji,jj) - ( zdusk(ji,jj) - ztmd(ji,jj) )
                  ENDIF
               ELSE
                  zdawn(ji,jj) = ztmd(ji,jj) + 0.5
                  zdusk(ji,jj) = zdawn(ji,jj)
               ENDIF
             END DO  
         END DO  
         zdawn(:,:) = MOD((zdawn(:,:) + 1.), 1.)
         zdusk(:,:) = MOD((zdusk(:,:) + 1.), 1.)


         !     2.2 Compute the scalling function:
         !         S* = the inverse of the time integral of the diurnal cycle from dawm to dusk

         DO jj = 1, jpj
            DO ji = 1, jpi
               IF ( ABS(zab(ji,jj)) < 1 ) THEN
         ! day duration is less than 24h
                  IF ( zdawn(ji,jj) < zdusk(ji,jj) ) THEN
         ! day time in one part
                     zscal(ji,jj) = fintegral(zdawn(ji,jj), zdusk(ji,jj), zaaa(ji,jj), zbbb(ji,jj), zccc(ji,jj)) 
                     zscal(ji,jj) = 1. / zscal(ji,jj)
                  ELSE
         ! day time in two parts
                     zscal(ji,jj) = fintegral(0., zdusk(ji,jj), zaaa(ji,jj), zbbb(ji,jj), zccc(ji,jj))   &
                        &         + fintegral(zdawn(ji,jj), 1., zaaa(ji,jj), zbbb(ji,jj), zccc(ji,jj)) 
                     zscal(ji,jj) = 1. / zscal(ji,jj)
                  ENDIF
               ELSE
                  IF ( zaaa(ji,jj) > zbbb(ji,jj) ) THEN
         ! 24h day
                     zscal(ji,jj) = fintegral(0., 1., zaaa(ji,jj), zbbb(ji,jj), zccc(ji,jj)) 
                     zscal(ji,jj) = 1. / zscal(ji,jj)
                  ELSE
         ! No day
                     zscal(ji,jj) = 0.
                  ENDIF
               ENDIF
            END DO  
         END DO  
         !
         ztmp = rday / rdt
         zscal(:,:) = zscal(:,:) * ztmp

      ENDIF 

         !     3. compute qsr with the diurnal cycle
         !     -----------------------

      DO jj = 1, jpj
         DO ji = 1, jpi
            IF ( ABS(zab(ji,jj)) < 1 ) THEN
         ! day duration is less than 24h
                  IF ( zdawn(ji,jj) < zdusk(ji,jj) ) THEN
         ! day time in one part
                     zlousd = MAX(zlo, zdawn(ji,jj))
                     zlousd = MIN(zlousd, zup)
                     zupusd = MIN(zup, zdusk(ji,jj))
                     zupusd = MAX(zupusd, zlo)
                     ztmp = fintegral(zlousd, zupusd, zaaa(ji,jj), zbbb(ji,jj), zccc(ji,jj)) 
                     pqsr(ji,jj) = qsr_daily(ji,jj) * ztmp * zscal(ji,jj)
                  ELSE
         ! day time in two parts
                     zlousd = MIN(zlo, zdusk(ji,jj))
                     zupusd = MIN(zup, zdusk(ji,jj))
                     ztmp1 = fintegral(zlousd, zupusd, zaaa(ji,jj), zbbb(ji,jj), zccc(ji,jj)) 
                     zlousd = MAX(zlo, zdawn(ji,jj))
                     zupusd = MAX(zup, zdawn(ji,jj))
                     ztmp2 = fintegral(zlousd, zupusd, zaaa(ji,jj), zbbb(ji,jj), zccc(ji,jj)) 
                     ztmp = ztmp1 + ztmp2
                     pqsr(ji,jj) = qsr_daily(ji,jj) * ztmp * zscal(ji,jj)
                  ENDIF
            ELSE
                  IF ( zaaa(ji,jj) > zbbb(ji,jj) ) THEN
         ! 24h day
                     ztmp = fintegral(zlo, zup, zaaa(ji,jj), zbbb(ji,jj), zccc(ji,jj)) 
                     pqsr(ji,jj) = qsr_daily(ji,jj) * ztmp * zscal(ji,jj)
                  ELSE
         ! No day
                     pqsr(ji,jj) = 0.
                  ENDIF
            ENDIF
         END DO  
      END DO  

   END SUBROUTINE sbc_dcy

   !!======================================================================
END MODULE sbcdcy
